
/******************************************************************
  (C) Copyright 2001 by UNITED BYTES
   
  web:  http://www.unitedbytes.de
        http://www.unitedbytes.org
		http://www.unitedbytes.net
  mail: info@unitedbytes.de

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, 
  Boston, MA  02111-1307, USA.

  *****************************************************************

  Program:      ubNetwork
  File:         sources/cub_n_socket.cpp
  Author:       Tim C. Schroeder, modified by Andreas Loeffler
  Date:         Mon, 10.04.2001
  Version:      1.0
  Description:  Simple socket class

  -----------------------------------------------------------------

  Changes:		-
  Date:			-
  Description:	-

  *****************************************************************/

#include "cub_n_socket.h"

// Static member variables
int CUB_Socket::m_iInstCout = 0;

//////////////////////////////////////////////////////////////////////
// Construcktion / destruction
//////////////////////////////////////////////////////////////////////

CUB_Socket::CUB_Socket()
{
	Init();
}

CUB_Socket::CUB_Socket(SOCKET iSocket)
{
	Init();
	m_sock = iSocket;
}

bool CUB_Socket::Init()
{
	//////////////////////////////////////////////////////////////////////
	// Init member variables
	//////////////////////////////////////////////////////////////////////

	m_sock = 0;

	m_iInstCout++;

	if (m_iInstCout == 1)
	{
	}

	return true;
}

CUB_Socket::~CUB_Socket()
{
	m_iInstCout--;

	// Close the socket (if opened)
	CloseSocket();
}

bool CUB_Socket::CreateSocket()
{
	//////////////////////////////////////////////////////////////////////
	// Create the socket
	//////////////////////////////////////////////////////////////////////

	if (!m_sock)
	{
		m_sock = socket(AF_INET, SOCK_STREAM, IPPROTO_IP);
		if (m_sock == SOCKET_ERROR)
			return false;
		return true;
	}

	return false;
}

bool CUB_Socket::CloseSocket()
{
	//////////////////////////////////////////////////////////////////////
	// Close the socket
	//////////////////////////////////////////////////////////////////////
	
	if (m_sock)
	{
		closesocket(m_sock);
		m_sock = 0;
		return true;
	}
	
	return true;
}

bool CUB_Socket::Connect(char szIPAddress[], unsigned int iPort)
{
	char cBuffer [1024];

	//////////////////////////////////////////////////////////////////////
	// Connect to the given host
	//////////////////////////////////////////////////////////////////////

	// Don't connect if no IP is present
	if (szIPAddress[0] == '\0')
		return false;

	if (iPort <= 0)
		return false;
		
	//////////////////////////////////////////////////////////////////////
	// Open the connection on the socket
	//////////////////////////////////////////////////////////////////////

	// Get the IP
	ResolveHost (szIPAddress, cBuffer);

	m_sa.sin_family           = AF_INET;
	m_sa.sin_port             = htons(iPort);
	m_sa.sin_addr.S_un.S_addr = inet_addr(cBuffer);

	// Call "connect" with passed IP address and the passed port
	if (connect(m_sock, (SOCKADDR *) &m_sa, sizeof (m_sa)) == SOCKET_ERROR)
		return false;

	return true;
}

bool CUB_Socket::ResolveHost (char *cIP, char *cResponseBuffer)
{
	int		iError = 0;
	struct	in_addr  sInAddr;
	struct  hostent *spHe = gethostbyname (cIP);

	// No IP returned ?
	if (!spHe)
	{
		if (cResponseBuffer != NULL) {
			strcpy(cResponseBuffer, "CUB_Server->ResolveHost: Host is unknown!");
			iError = 1;
		}
	}
	else {
		// Extract the IP and replace the host name with it
		memcpy (&sInAddr, spHe->h_addr_list[0], sizeof(struct in_addr));

		if (cResponseBuffer != NULL)
			strcpy (cResponseBuffer, inet_ntoa(sInAddr));
	}

	if (iError == 1)
		return false;

	return true;
}

bool CUB_Socket::Bind (unsigned int iPort, bool bListen)
{
	//////////////////////////////////////////////////////////////////////
	// Bind the socket to the given port
	//////////////////////////////////////////////////////////////////////

	// Disconnect and close if a connection / socket is alive
	if (m_sock != 0)
		CloseSocket();
	
	// Create a new socket
	if (!CreateSocket())
		return false;

	m_sa.sin_addr.s_addr = htonl(INADDR_ANY);
	m_sa.sin_family = AF_INET;
	m_sa.sin_port = htons(iPort);

	// Bind 
	if (bind(m_sock, (struct sockaddr *) &m_sa, sizeof(m_sa)) == SOCKET_ERROR)
		return false;

	if (bListen) 
	{
		// Set backlog
		if (listen(m_sock, 5) != 0)
			return false;
	}

	return true;
}

bool CUB_Socket::CanAccept()
{
	//////////////////////////////////////////////////////////////////////
	// Returns true if there is at least one incomming connection request
	//////////////////////////////////////////////////////////////////////

	struct timeval sTimeVal;
	fd_set		   sFd;
	int			   iRv;
	
	FD_ZERO (&sFd);
	FD_SET  (m_sock, &sFd);
	
	sTimeVal.tv_sec = 0;
	sTimeVal.tv_usec = 1;
	
	iRv = select (1, &sFd, NULL, NULL, &sTimeVal);
	
	return iRv ? true : false;
}

CUB_Socket *CUB_Socket::Accept(sockaddr *addr, int *addrlen)
{
	//////////////////////////////////////////////////////////////////////
	// Accepts incoming connections on the socket and returns the socket
	//////////////////////////////////////////////////////////////////////

	SOCKET ClientSocket;
	CUB_Socket *pSocket = 0;

	if (!m_sock)
		return 0;

	// Accept connection
	ClientSocket = accept(m_sock, addr, addrlen);

	// Connection established ?
	if (ClientSocket == INVALID_SOCKET)
		return 0;
	
	// Create new socket class to connect to the client
	pSocket = new CUB_Socket(ClientSocket);

	return pSocket;
}

bool CUB_Socket::ClearSocketBuffer ()
{
	//////////////////////////////////////////////////////////////////////
	// Clears the complete socket buffer
	//////////////////////////////////////////////////////////////////////

	int  iReturn;
	char cBuffer[255];

	if (!m_sock)
		return false;

	// If data old data in socket, clear it!
	while (Peek()) {
		iReturn = recv(m_sock, cBuffer, sizeof(cBuffer), 0);
	
		if (iReturn == 0 || iReturn == SOCKET_ERROR)
			break;
	}

	return true;
}

int CUB_Socket::Receive (char *cBuffer, long lBufLen)
{
	//////////////////////////////////////////////////////////////////////
	// Receive data from a connected socket
	//////////////////////////////////////////////////////////////////////

	int  iReturn;
		
	if (!m_sock)
		return false;

	if (!cBuffer)
		return false;

	// Clear the buffer
	memset (cBuffer, 0, sizeof(lBufLen));

	// Receive the data
	iReturn = recv(m_sock, cBuffer, lBufLen, 0);

	return iReturn;
}

bool CUB_Socket::Peek()
{
	//////////////////////////////////////////////////////////////////////
	// Return true if data is available on the connected socket
	//////////////////////////////////////////////////////////////////////

	int iReturn;
	char szBuffer[1];
		
	if (!m_sock)
		return 0;

	SetSocketBlockingMode (false);

	// Receive the data
	iReturn = recv(m_sock, szBuffer, sizeof(szBuffer), MSG_PEEK);

	SetSocketBlockingMode (true);

	// Error ?
	if (iReturn == 0 || iReturn == SOCKET_ERROR)
		return false;

	return true;
}

bool CUB_Socket::Wait(long lNetworkEvents, DWORD dwTimeOut)
{
/*	//////////////////////////////////////////////////////////////////////
	// Wait for one of of the lNetworkEvents to become signaled
	//////////////////////////////////////////////////////////////////////
	
	WSAEVENT hEvent;
	DWORD dwReturn;

	if (!m_sock)
		return false;
	
	// Create the event object
	hEvent = WSACreateEvent();

	// Let hEvent be triggered by lNetworkEvents
	if (WSAEventSelect(m_sock, hEvent, lNetworkEvents) == SOCKET_ERROR)
		return false;

	// Wait for the event(s)
	dwReturn = WSAWaitForMultipleEvents(1, &hEvent, FALSE, dwTimeOut, FALSE);

	// Close the event
	WSACloseEvent(hEvent);

	// Evaluate the return value
	if (dwReturn == WSA_WAIT_TIMEOUT ||
		dwReturn == WSA_WAIT_FAILED)
	{
		// Wait has failed
		return false;
	}*/

	// Event triggered
	return true;
}

int CUB_Socket::Send (char *cBuffer, long lBufLen)
{
	//////////////////////////////////////////////////////////////////////
	// Send data over a connected socket
	//////////////////////////////////////////////////////////////////////

	int iReturn = 0;
		
	if (!m_sock)
		return 0;

	// Send the data
	iReturn = send (m_sock, cBuffer, lBufLen, 0);

	// Error ?
	if (iReturn == SOCKET_ERROR)
		return false;

	return iReturn;
}

bool CUB_Socket::GetLocalIP(char szIPAddress[])
{
	//////////////////////////////////////////////////////////////////////
	// Return the IP adress of the last local interface. This is most
	// likely the PPP adapter (if connected)
	//////////////////////////////////////////////////////////////////////

	char ac[80];
	struct in_addr addr;
	unsigned int i;

	// Get the local host name
	if (gethostname(ac, sizeof(ac)) == SOCKET_ERROR) 
		return false;
    
	struct hostent *phe = gethostbyname(ac);
	
	// Test up to 64 addresses
	for (i=0; i<64; i++) 
	{
		// Last IP found ?
		if (phe->h_addr_list[i+1] == 0)
		{
			// Return
			memcpy(&addr, phe->h_addr_list[i], sizeof(struct in_addr));
			strcpy(szIPAddress, inet_ntoa(addr));
			break;
		}
    }
    
	return true;
}

bool CUB_Socket::GetRemoteIP(char szIPAddress[])
{
	//////////////////////////////////////////////////////////////////////
	// Return the IP adress of the peer
	//////////////////////////////////////////////////////////////////////

	struct sockaddr_in name;
	int iLen;

	if (!m_sock)
		return false;

	// Size of the sockaddr_in structure 
	iLen = sizeof(sockaddr_in);

	// Retrieve the peer name
	if (getpeername(m_sock, (sockaddr *) &name, &iLen) == SOCKET_ERROR)
		return false;
	
	// Copy the string into the passed buffer
	strcpy(szIPAddress, inet_ntoa(name.sin_addr));
	    
	return true;
}

void CUB_Socket::ResolveHost(char szHost[])
{
	//////////////////////////////////////////////////////////////////////
	// Resolve the passed host to an IP address
	//////////////////////////////////////////////////////////////////////

	struct in_addr addr;
	struct hostent *phe = gethostbyname(szHost);

	// No IP returned ?
	if (!phe)
	{
		strcpy(szHost, "<Unknown>");
		return;
	}

	// Extract the IP and replace the host name with it
	memcpy(&addr, phe->h_addr_list[0], sizeof(struct in_addr));
	strcpy(szHost, inet_ntoa(addr));
}

int CUB_Socket::Select(bool bRead, bool bWrite, int iTimeOut)
{
	//////////////////////////////////////////////////////////////////////
	// Determine the status of the connected socket
	//////////////////////////////////////////////////////////////////////

	fd_set ReadSocket;
	fd_set WriteSocket;
	timeval TimeOut;
	int iReturn;

	// Set timeout
	TimeOut.tv_sec = iTimeOut;			// Old: / 1000;
	TimeOut.tv_usec = 0;

	// Fill socket sets
	ReadSocket.fd_array[0] = m_sock;
	ReadSocket.fd_count = 1;
	WriteSocket.fd_array[0] = m_sock;
	WriteSocket.fd_count = 1;

	// Make the requested select call
	iReturn = select(0, bRead ? &ReadSocket : 0, bWrite ? &WriteSocket : 0,
		0, &TimeOut);

	if (iReturn == SOCKET_ERROR)
		return 0;

	return iReturn;
}

bool CUB_Socket::CanReceive ()
{
	//////////////////////////////////////////////////////////////////////
	// Determine the status of the connected socket
	//////////////////////////////////////////////////////////////////////

	fd_set  lReceiveSocket;
	timeval TimeOut;

	lReceiveSocket.fd_count    = 1;
	lReceiveSocket.fd_array[0] = m_sock;
		
	TimeOut.tv_sec = 0;
	TimeOut.tv_usec = 0;

	// Determine status of the socket
	if (select (0, &lReceiveSocket, NULL, NULL, &TimeOut) == 1)
		return true;
	else
		return false;
}

bool CUB_Socket::SetSocketBlockingMode (bool bBlocking)
{
	//////////////////////////////////////////////////////////////////////
	// Turns the blocking mode on/off
	//////////////////////////////////////////////////////////////////////

	unsigned long lMode = 0;

	if (bBlocking)
		lMode = 0;
	else
		lMode = 1;

    if (ioctlsocket (m_sock, FIONBIO, &lMode) == SOCKET_ERROR)
        return false;
	
	return true;
}  

void CUB_Socket::GetLastError(char *szErrDescription)
{
	//////////////////////////////////////////////////////////////////////
	// Return a string for the last error
	//////////////////////////////////////////////////////////////////////

	// Retrieve the last error
	int iError = WSAGetLastError();

	switch (iError)
	{
		case 0:                  
			strcpy(szErrDescription, "No error");
			break;
		case WSAEINTR:           
			strcpy(szErrDescription, "Interrupted system call");
			break;
		case WSAEBADF:           
			strcpy(szErrDescription, "Bad file number");
			break;
		case WSAEACCES:          
			strcpy(szErrDescription, "Permission denied");
			break;
		case WSAEFAULT:          
			strcpy(szErrDescription, "Bad address");
			break;
		case WSAEINVAL:          
			strcpy(szErrDescription, "Invalid argument");
			break;
		case WSAEMFILE:          
			strcpy(szErrDescription, "Too many open sockets");
			break;
		case WSAEWOULDBLOCK:     
			strcpy(szErrDescription, "Operation would block");
			break;
		case WSAEINPROGRESS:     
			strcpy(szErrDescription, "Operation now in progress");
			break;
		case WSAEALREADY:        
			strcpy(szErrDescription, "Operation already in progress");
			break;
		case WSAENOTSOCK:        
			strcpy(szErrDescription, "Socket operation on non-socket");
			break;
		case WSAEDESTADDRREQ:    
			strcpy(szErrDescription, "Destination address required");
			break;
		case WSAEMSGSIZE:        
			strcpy(szErrDescription, "Message too long");
			break;
		case WSAEPROTOTYPE:      
			strcpy(szErrDescription, "Protocol wrong type for socket");
			break;
		case WSAENOPROTOOPT:     
			strcpy(szErrDescription, "Bad protocol option");
			break;
		case WSAEPROTONOSUPPORT: 
			strcpy(szErrDescription, "Protocol not supported");
			break;
		case WSAESOCKTNOSUPPORT: 
			strcpy(szErrDescription, "Socket type not supported");
			break;
		case WSAEOPNOTSUPP:      
			strcpy(szErrDescription, "Operation not supported on socket");
			break;
		case WSAEPFNOSUPPORT:    
			strcpy(szErrDescription, "Protocol family not supported");
			break;
		case WSAEAFNOSUPPORT:    
			strcpy(szErrDescription, "Address family not supported");
			break;
		case WSAEADDRINUSE:      
			strcpy(szErrDescription, "Address already in use");
			break;
		case WSAEADDRNOTAVAIL:   
			strcpy(szErrDescription, "Can't assign requested address");
			break;
		case WSAENETDOWN:        
			strcpy(szErrDescription, "Network is down");
			break;
		case WSAENETUNREACH:     
			strcpy(szErrDescription, "Network is unreachable");
			break;
		case WSAENETRESET:       
			strcpy(szErrDescription, "Net connection reset");
			break;
		case WSAECONNABORTED:    
			strcpy(szErrDescription, "Software caused connection abort");
			break;
		case WSAECONNRESET:      
			strcpy(szErrDescription, "Connection reset by peer");
			break;
		case WSAENOBUFS:         
			strcpy(szErrDescription, "No buffer space available");
			break;
		case WSAEISCONN:         
			strcpy(szErrDescription, "Socket is already connected");
			break;
		case WSAENOTCONN:        
			strcpy(szErrDescription, "Socket is not connected");
			break;
		case WSAESHUTDOWN:       
			strcpy(szErrDescription, "Can't send after socket shutdown");
			break;
		case WSAETOOMANYREFS:    
			strcpy(szErrDescription, "Too many references, can't splice");
			break;
		case WSAETIMEDOUT:       
			strcpy(szErrDescription, "Connection timed out");
			break;
		case WSAECONNREFUSED:    
			strcpy(szErrDescription, "Connection refused");
			break;
		case WSAELOOP:           
			strcpy(szErrDescription, "Too many levels of symbolic links");
			break;
		case WSAENAMETOOLONG:    
			strcpy(szErrDescription, "File name too long");
			break;
		case WSAEHOSTDOWN:       
			strcpy(szErrDescription, "Host is down");
			break;
		case WSAEHOSTUNREACH:   
			strcpy(szErrDescription, "No route to host");
			break;
		case WSAENOTEMPTY:       
			strcpy(szErrDescription, "Directory not empty");
			break;
		case WSAEPROCLIM:        
			strcpy(szErrDescription, "Too many processes");
			break;
		case WSAEUSERS:          
			strcpy(szErrDescription, "Too many users");
			break;
		case WSAEDQUOT:          
			strcpy(szErrDescription, "Disc quota exceeded");
			break;
		case WSAESTALE:          
			strcpy(szErrDescription, "Stale NFS file handle");
			break;
		case WSAEREMOTE:         
			strcpy(szErrDescription, "Too many levels of remote in path");
			break;
		case WSASYSNOTREADY:     
			strcpy(szErrDescription, "Network system is unavailable");
			break;
		case WSAVERNOTSUPPORTED: 
			strcpy(szErrDescription, "Winsock version out of range");
			break;
		case WSANOTINITIALISED:  
			strcpy(szErrDescription, "WSAStartup not yet called");
			break;
		case WSAEDISCON:         
			strcpy(szErrDescription, "Graceful shutdown in progress");
			break;
		default:
			strcpy(szErrDescription, "Unknown error");
			break;
	}
}
